/* Jesus Christ is Lord! */
/* FUTOCALL.C - A program designed to create a Callibro formatted log file */
/* from the QSO.DAT file created by Jeff Ach's VHFTEST.                    */
/* Version 1.4 - Created 01-22-2002     */
/* Revision History                     */
/* 01-17-01 1.0: Inital release, barely beyond beta  */
/* 01-18-01 1.1: Repaired a bug that caused the band '50 MHz' to be */
/*               occasionally not recognized. Also fixed a minor bug */
/*               that caused some very uncollon DX call formats to be */
/*               invalid. */
/* 01-21-01 1.2  Corrected yet another bug in the band recognition */
/*               section. This was due to thinking that some microwave */
/*               bands had abbreviated names, when in fact they did not. */
/* 09-20-01 1.3  Corrected a bug that was causing single day dates to be */
/*               printed with a cryptic leading character. */
/* 01-22-02 1.4  Added a semicolon to the 'END-OF-FILE' field to bring */
/*               output up to date with latest ARRL standards.  */

#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <graph.h>
#include <string.h>

#undef toupper

struct qsorec
{
char band;
char mode[2];
char date[2];
char time[4];
char txgrid[4];
char rxcall[12];
char rxgrid[4];
char month[2];
char year[4];
};

int qsoparse(FILE *qsodat, struct qsorec *qrecptr, int recnum, int modeflag);
int callchk(char call[12], int calllen);
int gridchk(char grid[4]);
void clrscrn(void);
int warnmsg(int errortype, int recnum);
int record(char ch, FILE *cabptr, struct qsorec *qrecptr);
int recstr(char *workptr, int c, FILE *cabptr, struct qsorec *qrecptr);
int crlf(FILE*cabptr, struct qsorec *qrecptr);

void main()
{

/* Initalize variables */



int l, w, x, y, z;
int ucalllen, seclen, contesttype, class, numoplines, numaddrlines;
int numsoaplines, recnum, numrec;
unsigned int p;
size_t b;
char usrcall[12], usrcallin[13];
char section[3], sectionin[4], name[40], club[40], operators[3][81];
char address[4][51], soapbox[4][81], outtitle[12], works[50], score[9];
double a;
struct qsorec *qrecptr;
FILE *qsoptr, *cabptr;

/* Banner print */

clrscrn();
puts("                   VHFTEST VHF CONTEST LOGGING SOFTWARE");
puts("                           CABRILLO FILE FORMAT");
puts("                             RESULT GENERATOR");
puts("                                 Ver 1.3\n");
puts("This program takes the QSO.BAT file of the W2FU VHFTEST VHF contest");
puts("logging and uses it to create a log file in the ARRL required");
puts("Cabrillo format. If you are multiop, before running this program,");
puts("make sure you have combined your logs into one QSO.DAT file, and");
puts("it is in chronological order. Otherwise, this program requires no");
puts("preparation to run. Note that this program does a thorough check on");
puts("the integrity of the QSO.DAT file. It will flag suspicious calls and");
puts("grids, giving you a chance to either verify they are correct or fix");
puts("them if they are wrong. The program will also halt on an obvious");
puts("data inconsistency in QSO.DAT. So, just answer the questions below,");
puts("and your file will be automatically genertaed. The file will take");
puts("the name <yourcall>.LOG.\n");

/* Clear STDIN for data entry */

fflush(stdin);

/* Test QSO.DAT for size and the first few records for integrity. */

if((qsoptr = fopen("QSO.DAT","rb")) == NULL) {
 warnmsg(20, 0);
 exit(0);
 }

a = -1;
while(feof(qsoptr) == 0x00) {
 l = fgetc(qsoptr);
 a++;
 }

if(a < 54) {
 warnmsg(21, 0);
 fclose(qsoptr);
 exit(0);
 }
if(fmod(a,54) > 0) {
 warnmsg(21, 0);
 fclose(qsoptr);
 exit(0);
 }

b =(size_t)(a/54);
numrec = (int)b;

/* Allocate memory for processed QSO array */
qrecptr = (struct qsorec *)calloc(b, sizeof(struct qsorec));
if(qrecptr == NULL) {
 warnmsg(22, 0);
 fclose(qsoptr);
 exit(0);
 }

rewind(qsoptr);
for(x = 0; x <= 3; x++) {
 w = qsoparse(qsoptr, qrecptr, 0, 0);
 if(w > 0) {
  y = warnmsg(w, 0);
  if(y == 1) {
   fclose(qsoptr);
   free(qrecptr);
   exit(0);
   }
  }
 }

/* Close QSO.DAT for a while. */

if(fclose(qsoptr)) {
 warnmsg(25, 0);
 free(qrecptr);
 exit(0);
 }

puts("QSO.DAT appears to be in good condition!");

/* Get miscellaneous data */

/* User's call */

x = 3;
do {
 puts("Please enter the call your station used");
 gets(usrcallin);
 ucalllen = strlen(usrcallin);
 if(ucalllen > 12)
  ucalllen = 12;
 for(z = 0; z <= 11; z++)
  usrcall[z] = 0x20;
 for(z = 0; z <= (ucalllen-1); z++)
  usrcall[z] = (char)toupper(usrcallin[z]);
 x = callchk(usrcall, ucalllen);
 if(x == 1) {
  puts("DX or unusual call!");
  x = 0;
  }
 if(x == 2)
  puts("Invalid call, please try again!");
 }while(x != 0);

/* ARRL section */

x = 0;
do
 {
 puts("Please enter your 2 or 3 letter ARRL section (Ex. WNY)");
 gets(sectionin);
 seclen = strlen(sectionin);
 if(seclen > 3)
  seclen = 3;
 for(y = 0; y <= 2; y++)
  section[y] = 0x20;
 if(seclen > 1) {
  for(z = 0; z <= (seclen -1); z++)
   section[z] = (char)toupper(sectionin[z]);
  x = 1;
  }
 else
  puts("Section appears to have too few characters (Must be 2 or 3");
}while(x == 0);

/* Get contest */

contesttype = 0;
puts("Please enter the number of the contest you operated:");
puts(" 1. ARRL January VHF Contest.");
puts(" 2. ARRL June VHF QSO Party.");
puts(" 3. ARRL August UHF Contest.");
puts(" 4. ARRL September VHF QSO Party.");
do {
 scanf("%i", &contesttype);
 if(contesttype < 1 || contesttype > 4) {
  puts("Invalid choice. Must be [1-4]. Try again!");
  x = 0;
  }
 else
  x = 1;
 }while(x == 0);

/* Get contest class */
x = 0;
class = 0;
puts("Please enter the number of the class you entered:");
puts(" 1. Single operator, QRP portable.");
puts(" 2. Single operator, Fixed, Low power.");
puts(" 3. Single operator, Fixed, High power.");
puts(" 4. Limited multioperator.");
puts(" 5. Full multioperator.");
puts(" 6. Rover.");
puts(" 7. Checklog only.");
do {
 scanf("%i", &class);
 if(class < 1 || class > 7) {
  puts("Invalid choice. Must be [1-6]. Try again!");
  x = 0;
  }
 else
  x = 1;
 }while(x == 0);

/* Get operator calls. */

puts("On many of the following entries (Operator calls, Address, Soapbox),");
puts("Multiple lines of text may be accepted. There is no word wrap, etc.,");
puts("so keep your lines under 80 characters. Press 'Enter' on a blank");
puts("line to end input on these entries.\n");

fflush(stdin);
numoplines = 0;
if(class > 3 && class < 7) {
 puts("Please enter the calls of all operators who particiapted in your");
 puts("operation. Seperate each call with a space. (Limit three lines.)");
 for(z = 0; z <= 2; z++) {
  gets(operators[z]);
  if(strlen(operators[z]) == 0)
   break;
  }
 numoplines = z;

 for(z = 0; z <= (numoplines - 1); z++)
  for(p = 0; p <= (strlen(operators[z]) - 1); p++)
   operators[z][p] = (char)toupper(operators[z][p]);
 }
else {
 puts("Please enter the call of the operator who operated the above");
 puts("station, even if it is the same operator.");
 gets(operators[0]);
 for(p = 0; p <= strlen(operators[0] - 1); p++)
  operators[0][p] = (char)toupper(operators[0][p]);
 numoplines = 1;
 }

/* Get club name */

puts("Please enter club name for club competition, if needed (40");
puts("characters max.");
gets(club);

/* Get claimed score */

puts("Please enter your CLAIMED SCORE!");
gets(score);
fflush(stdin);

/* Submitter's name */

puts("Please enter your name. (40 characters max.)");
gets(name);

/* Submitter's address */

numaddrlines = 0;
puts("Please enter your mailing address. (4 lines max, 50 char/line.)");
 for(z = 0; z <= 3; z++) {
  gets(address[z]);
  if(strlen(address[z]) == 0)
   break;
  }
 numaddrlines = z;

/* Soapbox entry */

numsoaplines = 0;
puts("Please enter any soapbox comments you wish to add. ( 4 lines max.,");
puts("50 char/line.) NOTE: You may add more soapbox comments by editing");
puts("the result file in any text editor. (Use Edit, Notepad, EMACS, etc;");
puts("not a word processor!) You will see how the lines are to be formatted");
puts("by looking at the existing lines. Of course, you may enter no");
puts("comments by simply pressing 'enter' on a blank line.");
for(z = 0; z <= 3; z++) {
 gets(soapbox[z]);
 if(strlen(soapbox[z]) == 0)
  break;
 }
numsoaplines = z;

puts("*** Data entry complete! Building Cabrillo log. **");

/* Build data structure */

for(x = 0; x <=(numrec -1); x++) {
 for(y = 0; y <= 11; y++)
  qrecptr[x].rxcall[y] = 0x00;
  }

if((qsoptr = fopen("QSO.DAT","rb")) == NULL) {
 warnmsg(20, 0);
 exit(0);
 }

for(recnum = 0; recnum <=(numrec-1); recnum++) {
 w = qsoparse(qsoptr, qrecptr, recnum, 1);
 if(w > 0) {
  x = warnmsg(w,recnum);
  if(x == 1) {
   fclose(qsoptr);
   free(qrecptr);
   exit(0);
   }
  }
 }

if(fclose(qsoptr)) {
 warnmsg(25, 0);
 free(qrecptr);
 exit(0);
 }


/* Whew! The worst part is finished! */

/* Write the Cabrillo file */

puts("QSO.DAT sucessfully processed. Writing output file");

for(x = 0; x <= 11; x++)
 outtitle[x] = 0x20;

for(x = 0; x <= (ucalllen -1); x++) {
 outtitle[x] = usrcall[x];
 if(usrcall[x+1] == 0x2f)
  break;
 if(x == 5)
  break;
 }
x++;
outtitle[x] = 0x00;
strcat(outtitle,".LOG");

if((cabptr = fopen(outtitle,"wb")) == NULL) {
 warnmsg(23, 0);
 exit(0);
 }

/* Write header section of output file. */

strcpy(works, "START-OF-LOG: 2.0");
recstr(works, 17, cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "CREATED-BY: NS9E VHFTEST Cabrillo converter 1.3");
recstr(works, 47, cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works,"CONTEST: ");

if(contesttype == 1)
 strcat(works, "ARRL-VHF-JAN");

if(contesttype == 2)
 strcat(works, "ARRL-VHF-JUN");

if(contesttype == 3)
 strcat(works, "ARRL-VHF-AUG");

if(contesttype == 4)
 strcat(works, "ARRL-VHF-SEP");

recstr(works, 21, cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "CALLSIGN: ");
strncat(works, usrcall, ucalllen);
recstr(works, (10 + ucalllen), cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "ARRL-SECTION: ");
strncat(works, section, seclen);
recstr(works, (14 + seclen), cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "CATEGORY: ");
switch(class)
{

 case 1:
  strcat(works, "SINGLE-OP-QRP-PORTABLE ALL QRP");
  w = 40;
  break;

 case 2:
  strcat(works, "SINGLE-OP ALL LOW");
  w = 27;
  break;

 case 3:
  strcat(works, "SINGLE-OP ALL HIGH");
  w = 28;
  break;

 case 4:
  strcat(works, "MULTI-LIMITED ALL HIGH");
  w = 32;
  break;

 case 5:
  strcat(works, "MULTI-UNLIMITED ALL HIGH");
  w = 34;
  break;

 case 6:
  strcat(works, "ROVER ALL HIGH");
  w = 24;
  break;

 case 7:
  strcat(works, "CHECKLOG");
  w = 18;
  break;

 default:
  warnmsg(16, 0);
  fclose(cabptr);
  free(qrecptr);
  exit(0);

}
recstr(works, w, cabptr, qrecptr);
crlf(cabptr, qrecptr);

if(numoplines >0)
 for(x = 0; x <= (numoplines -1); x++) {
  strcpy(works, "OPERATORS: ");
  recstr(works, 11, cabptr, qrecptr);
  if(strlen(operators[x]) > 0)
   recstr(operators[x], strlen(operators[x]), cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }
else {
  strcpy(works, "OPERATORS: ");
  recstr(works, 11, cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }


strcpy(works, "CLUB: ");
recstr(works, 6, cabptr, qrecptr);
if(strlen(club) != 0)
 recstr(club, strlen(club), cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "CLAIMED-SCORE: ");
recstr(works, 15, cabptr, qrecptr);
if(strlen(score) != 0)
 recstr(score, strlen(score), cabptr, qrecptr);
crlf(cabptr, qrecptr);

strcpy(works, "NAME: ");
recstr(works, 6, cabptr, qrecptr);
if(strlen(name) != 0)
 recstr(name, strlen(name), cabptr, qrecptr);
crlf(cabptr, qrecptr);

if(numaddrlines >0)
 for(x = 0; x <= (numaddrlines - 1); x++) {
  strcpy(works, "ADDRESS: ");
  recstr(works, 9, cabptr, qrecptr);
  if(strlen(address[x]) > 0)
   recstr(address[x], strlen(address[x]), cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }
else
 for(x = 0; x <= 2; x++) {
  strcpy(works, "ADDRESS: ");
  recstr(works, 9, cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }

if(numsoaplines >0)
 for(x = 0; x <= (numsoaplines - 1); x++) {
  strcpy(works, "SOAPBOX: ");
  recstr(works, 9, cabptr, qrecptr);
  if(strlen(soapbox[x]) > 0)
   recstr(soapbox[x], strlen(soapbox[x]), cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }
else
 for(x = 0; x <= 3; x++) {
  strcpy(works, "SOAPBOX: ");
  recstr(works, 9, cabptr, qrecptr);
  crlf(cabptr, qrecptr);
  }

/* Write QSO information */

puts("Now writing QSO information.");

for(recnum = 0; recnum <= (numrec -1); recnum++)
{
strcpy(works, "QSO:     ");
recstr(works, 9, cabptr, qrecptr);
record(qrecptr[recnum].band, cabptr, qrecptr);
record(' ', cabptr, qrecptr);
recstr(qrecptr[recnum].mode, 2, cabptr, qrecptr);
record(' ', cabptr, qrecptr);
recstr(qrecptr[recnum].year, 4, cabptr, qrecptr);
record('-', cabptr, qrecptr);
recstr(qrecptr[recnum].month, 2, cabptr, qrecptr);
record('-', cabptr, qrecptr);
recstr(qrecptr[recnum].date, 2, cabptr, qrecptr);
record(' ', cabptr, qrecptr);
recstr(qrecptr[recnum].time, 4, cabptr, qrecptr);
record(' ', cabptr, qrecptr);
recstr(usrcall, ucalllen, cabptr, qrecptr);
for(x = 1; x < (18 - ucalllen); x++)
 record(' ', cabptr, qrecptr);
recstr(qrecptr[recnum].txgrid, 4, cabptr, qrecptr);
for(x = 1; x <= 3; x++)
 record(' ', cabptr, qrecptr);
y = strlen(qrecptr[recnum].rxcall);
recstr(qrecptr[recnum].rxcall, y, cabptr, qrecptr);
for(x = 1; x < (18 - y); x++)
 record(' ', cabptr, qrecptr);
recstr(qrecptr[recnum].rxgrid, 4, cabptr, qrecptr);
crlf(cabptr, qrecptr);
}
strcpy(works, "END-OF-LOG:");
recstr(works, 11, cabptr, qrecptr);
crlf(cabptr, qrecptr);
record(0x1a, cabptr, qrecptr);

if(fclose(cabptr)) {
 warnmsg(26, 0);
 free(qrecptr);
 exit(0);
 }
else {
 puts("Logfile sucessfully written!!");
 free(qrecptr);
 exit(0);
 }

/* End of Main! */

}

/* qsoparse gets a record from the QSO.DAT file, and parses it into        */
/* individual data. In the process, it does a thorough check of the        */
/* data contained in the record. If the modeflg is set, it writes data to  */
/* the qso record structure.                                               */

int qsoparse(FILE *qsoptr, struct qsorec *qrecptr, int recnum, int modeflag)
{

unsigned char recbuffer[54];
char band, mode[2], date[2], month[2], call[12], grid[4];
int x, z, dxflag = 0, gridflag = 0, calllen;

 /* Read next record */
if(fread(recbuffer, sizeof(recbuffer), 1, qsoptr) != 1)
 if(feof(qsoptr))
  return(2);
 else
  return(15);

/* Check for proper placement of record size numbers */
if(recbuffer[0] != 0x04)
 return(3);

if(recbuffer[5] < 0x02 || recbuffer[5] > 0x04)
 return(3);

if(recbuffer[10] < 0x02 || recbuffer[10] > 0x03)
 return(3);

if(recbuffer[15] != 0x04)
 return(3);

if(recbuffer[20] != 0x04)
 return(3);

if(recbuffer[25] < 0x03 || recbuffer[25] > 0x0C)
 return(3);

if(recbuffer[38] != 0x04)
 return(3);

if(recbuffer[46] != 0x01 && recbuffer[46] != 0x02)
 return(3);

if(recbuffer[49] != 0x04)
 return(3);


/* Extract band and convert to new format */

z = 1;
if(recbuffer[z] == 0x20)
 z = z + 1;
if(recbuffer[z] == 0x20)
 z = z + 1;
if(recbuffer[z] == 0x20)
 return(4);

switch(recbuffer[z])
 {
 case '1':
    if(recbuffer[z+1] == '4') {
     band = 'B';
     break;
     }
    else
     if(recbuffer[z+1] == '2' || recbuffer[z + 1] == 0x2e) {
      band = 'E';
      break;
      }
     else
      if(recbuffer[z+1] == '0') {
       band = 'I';
       break;
       }
      else
       return(4);

 case '2':
     if(recbuffer[z+1] == '2') {
      band ='C';
      break;
      }
     else
       if(recbuffer[z+1] == '3' || recbuffer[z + 1] == 0x2e) {
	band = 'F';
	break;
	}
       else
	if(recbuffer[z+1] == '4') {
	 band = 'J';
	 break;
	 }
	else
	 return(4);

 case '3':
     if(recbuffer[z+1] == '4' || recbuffer[z + 1] == 0x2e) {
      band = 'G';
      break;
      }
     else
      return(4);

 case '4':
     if(recbuffer[z+1] == '3') {
      band = 'D';
      break;
      }
     else
      if(recbuffer[z+1] == '7') {
       band = 'K';
       break;
       }
      else
       return(4);

 case '5':
     if(recbuffer[z+1] == '0') {
      band = 'A';
      break;
      }
     else
      if(recbuffer[z+1] == '7' || recbuffer[z + 1] == 0x2e) {
       band = 'H';
       break;
       }
      else
       return(4);

 case '9':
     if(recbuffer[z+1] == '0') {
      band = '9';
      break;
      }
     else
      return(4);

 case 'L':
     if(recbuffer[z+1] == 'I' || recbuffer[z+1] == 'i') {
      band = 'L';
      break;
      }
     else
      return(4);

 default:
     return(4);
 }

/* Determine mode. */

z = 6;
if(recbuffer[z] == 0x20)
 z = z + 1;

switch(recbuffer[z])
 {
 case 'S':
  if(recbuffer[z+1] == 'S') {
   mode[0] = 'P';
   mode[1] = 'H';
   break;
   }
  else
   return(5);

 case 'F':
  if(recbuffer[z+1] == 'M') {
   mode[0] = 'F';
   mode[1] = 'M';
   break;
   }
  else
   return(5);

 case 'C':
  if(recbuffer[z+1] == 'W') {
   mode[0] = 'C';
   mode[1] = 'W';
   break;
   }
  else
   return(5);

 default:
  return(5);
 }

/* Determine date */

if(recbuffer[10] == 0x02)
 if(recbuffer[11] >= 0x31 && recbuffer[11] <= 0x39) {
  date[0] = 0x30;
  date[1] = recbuffer[11];
  }
 else
  return(6);

if(recbuffer[10] == 0x03)
 if(recbuffer[11] >= 0x31 && recbuffer[11] <= 0x33 && recbuffer[12] >= 0x30
 && recbuffer[12] <= 0x39) {
   date[0] = recbuffer[11];
   date[1] = recbuffer[12];
   }
 else
  return(6);

/* Check time */

for(x = 16; x <= 19; x++)
  if(recbuffer[x] < 0x30)
   return(7);

if(recbuffer[17] > 0x39 || recbuffer[19] > 0x39)
 return(7);
if(recbuffer[16] > 0x32)
 return(7);
if(recbuffer[16] > 0x35)
 return(7);

/* Check RX and TX grid */

for(z = 0; z <= 3; z++)
 grid[z] = recbuffer[z + 21];
if(gridchk(grid) == 2)
 return(8);

for(z = 0; z <= 3; z++)
 grid[z] = recbuffer[z + 39];
x = gridchk(grid);
if(x == 1)
 gridflag = 1;
if(x == 2)
 return(10);

/* Check call */

for(x = 0; x <= 11; x++)
 call[x] = 0x20;
calllen = (int)recbuffer[25];
for(x = 0; x <= calllen - 1; x++)
 call[x] = recbuffer[26 + x];
z = callchk(call, calllen);
if(z == 1)
 dxflag = 1;
if(z == 2)
 return(9);

/* Check month */

if(recbuffer[46] == 0x01)
 if(recbuffer[47] >= 0x31 && recbuffer[11] <= 0x39) {
  month[0] = 0x30;
  month[1] = recbuffer[47];
  }
 else
  return(13);

if(recbuffer[46] == 0x02)
 if(recbuffer[47] == 0x31)
  if(recbuffer[48] >= 0x30 && recbuffer[48] <=0x32) {
   month[0] = recbuffer[47];
   month[1] = recbuffer[48];
   }
  else
   return(13);
 else
  if(recbuffer[47] == 0x20)
   if(recbuffer[48] >= 0x31 && recbuffer[12] <=0x39) {
    month[0] = 0x30;
    month[1] = recbuffer[48];
    }
   else
    return(13);

/* Check year */

if(recbuffer[50] < '1' || recbuffer[50] > '2')
 return(14);
for(x = 51; x <= 53; x++)
 if(recbuffer[x] < '0' || recbuffer[x] > '9')
  return(14);

/* If we got this far data is good enough to write a record for Callibro.  */
/* First, we will check to see if this is just a consistency check, or if  */
/* a record is actually to be written.                                     */

if(modeflag == 0) {
 if(dxflag == 1 && gridflag == 0)
  return(1);
 if(dxflag == 0 && gridflag == 1)
  return(11);
 if(dxflag == 1 && gridflag == 1)
  return(12);
 return(0);
 }

/* Write data to QSO structure */

qrecptr[recnum].band = band;

for(x = 0; x <= 1; x++)
 qrecptr[recnum].mode[x] = mode[x];

for(x = 0; x <= 1; x++)
 qrecptr[recnum].date[x] = date[x];

for(x = 0; x <= 1; x++)
 qrecptr[recnum].month[x] = month[x];

for(x = 0; x <= (calllen - 1); x++)
 qrecptr[recnum].rxcall[x] = recbuffer[26 + x];

for(x = 0; x <= 3; x++)
 qrecptr[recnum].txgrid[x] = recbuffer[21 + x];

for(x = 0; x <= 3; x++)
 qrecptr[recnum].rxgrid[x] = recbuffer[39 + x];

for(x = 0; x <= 3; x++)
 qrecptr[recnum].time[x] = recbuffer[16 + x];

for(x = 0; x <= 3; x++)
 qrecptr[recnum].year[x] = recbuffer[50 + x];

if(dxflag == 1 && gridflag == 0)
 return(1);
if(dxflag == 0 && gridflag == 1)
 return(11);
if(dxflag == 1 && gridflag == 1)
 return(12);
return(0);
}

/* Gridchk checks the integrity of a gridsquare designation. It returns a  */
/* 2 upon a frank error, a 1 if the grid square is OK, but outside the     */
/* field range CL - FN, and a 0 otherwise.                                 */

int gridchk(char grid[4])
{
int flag = 0;
if(grid[0] < 'A' || grid[0] > 'Z')
 return(2);
if(grid[0] < 'C' || grid[0] > 'F')
 flag = 1;

if(grid[1] < 'A' || grid[1] > 'Z')
 return(2);
if(grid[1] < 'L' || grid[1] > 'N')
 flag = 1;

if(grid[2] < 0x30 || grid[2] > 0x39)
 return(2);

if(grid[3] < 0x30 || grid[3] > 0x39)
 return(2);

if(flag == 1)
 return(1);
else
 return(0);
}

/* Callchk tests a call to see if there is a reasonable chance it is a good */
/* call. If it conforms with W/VE call usage, the routhine returns a 0. If  */
/* at least the first four characters represent a possible valid call, but  */
/* is not a W/VE call, the routine returns a 1. All other cases represent a */
/* call that is most likely invalid, and they return a 2. If the minimum    */
/* requirements for a call are met, the program stops checking when it      */
/* encounters a forward slash character. However, subsequent characters are */
/* checked to verify that they are alphanumeric or a forward slash.         */

int callchk(char call[12], int calllen)
{
int x, flag1 = 0, flag2 = 0, flag3 = 0, dxflag = 0;

/* Check for minimum possible length-- here mainly for code re-use. */

if(calllen < 3)
 return(2);

/* First character check */

if(!((call[0] >= '0' && call[0] <= '9') || (call[0] >= 'A' &&
      call[0] <= 'Z')))
  return(2);

if(call[0] >= '0' && call[0] <= '9') {
 dxflag = 1;
 flag1 = 1;
 }

if(call[0] != 'A' && call[0] != 'C' && call[0] != 'K' && call[0] != 'N'
      && call[0] != 'V' && call[0] != 'W')
 dxflag = 1;

/* Second character check */

if(!((call[1] >= '0' && call[1] <= '9') || (call[1] >= 'A' &&
      call[1] <= 'Z')))
  return(2);

if(call[1] >= '0' && call[1] <= '9' && flag1 == 1)
 return(2);

if(call[1] >= '0' && call[1] <= '9')
 flag3 = 1;

if(call[1] >= 'A' && call[1] <= 'Z')
 flag2 = 1;

/* Third character check */

if(call[2] == 0x2f)
 return(1);

if(!((call[2] >= '0' && call[2] <= '9') || (call[2] >= 'A' &&
      call[2] <= 'Z')))
  return(2);

if((flag1 == 1 || flag2 == 1) && (call[2] < '0' || call[2] > '9'))
 return(2);

/* Fourth character check */

if((dxflag == 1 || flag1 == 1 || flag2 == 1) && calllen < 4)
 return(2);

if(calllen == 3)
 if(dxflag == 1)
  return(1);
 else
  return(0);

if((call[3] < 'A' || call[3] > 'Z') && call[3] != 0x2f)
 return(2);

if(call[3] == 0x2f)
 if(dxflag == 1)
  return(1);
 else
  return(0);

/* Fifth character check */

if(calllen == 4)
 if(dxflag == 1)
  return(1);
 else
  return(0);

if((call[4] < 'A' || call[4] > 'Z') && call[4] != 0x2f)
 return(2);

if(call[4] == 0x2f)
 if(dxflag == 1)
  return(1);
 else
  return(0);

 /* Sixth character check */

if(calllen == 5)
 if(dxflag == 1)
  return(1);
 else
  return(0);

if(flag3 == 1 && call[5] != 0x2f && dxflag == 0)
 return(2);

if((call[5] < 'A' || call[5] > 'Z') && call[5] != 0x2f)
 return 2;

if(call[5] == 0x2f)
 if(dxflag == 1)
  return(1);
 else
  return(0);

/* Seventh character check */

if(calllen == 6)
 if(dxflag == 1)
  return(1);
 else
  return(0);

/* all characters after the sixth must start with a forward slash. If we  */
/* got this far, and the seventh character is a slash, it is a legitimate */
/* call. The remainder of the call need just be checked for illegal       */
/* characters.                                                            */


if(call[6] != 0x2f)
 return(2);

for (x = 6; x <= (calllen-1); x++)
 if(!((call[x] >= '0' && call[x] <= '9') || (call[x] >= 'A' &&
      call[x] <= 'Z')) && call[x] != 0x2f)
 return(2);

if(dxflag == 1)
 return(1);
else
 return(0);
}

/* Warnmsg prints out warning messages. If it returns 0, the error is a     */
/* caution only. If returns a 1, the error is fatal, and the program should */
/* abort.   */

int warnmsg(int errortype, int recnum)
{

if(errortype == 0)
 puts("No error- I'm not supposed to be here!");

if(errortype == 1 || errortype == 12)
 printf("Check call in record number %d -- it may be a typo!\n", recnum);

if(errortype == 11 || errortype == 12)
 printf("Check grid in record number %d -- it may be a typo\n", recnum);

if(errortype == 2)
 puts("Unexpected End Of File in QSO.DAT");

if(errortype == 3)
 printf("Data structure of record %d QSO.DAT is not correct.\n",recnum);

if(errortype == 4)
 printf("Band not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 5)
 printf("Mode not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 6)
 printf("Date not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 7)
 printf("Time not valid in record %d of QSO.DAT.\n",recnum);

if(errortype == 8)
 printf("TX grid not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 9)
 printf("Call is not valid in record %d of QSO.DAT.\n",recnum);

if(errortype == 10)
 printf("RX grid not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 13)
 printf("Month not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 14)
 printf("Year not recognized in record %d of QSO.DAT.\n",recnum);

if(errortype == 15)
 puts("Error while reading QSO.DAT");

if(errortype == 16)
 puts("Undefined error while writing result file.");

if(errortype == 20)
 puts("Could not find or open QSO.DAT in this directory.");

if(errortype == 21)
 puts("QSO.DAT is not the correct size. Should be a multiple of 54 bytes.");

if(errortype == 22)
 puts("Could not allocate enough memory to build QSO data structure.");

if(errortype == 23)
 puts("Could not open the output file.");

if(errortype == 24)
 puts("Problems writing to the output file");

if(errortype == 25)
 puts("Could not close QSO.DAT");

if(errortype == 26)
 puts("Could not close output file");

if(errortype == 1 || errortype == 11 || errortype == 12) {
 puts("This is a caution message only. Processing is proceeding as normal.");
 puts("However, you may wish to check this item to see if it is really");
 puts("what is intended.\n");
 }

if((errortype > 1 && errortype < 11) || errortype > 12) {
 puts("FATAL ERROR. Program will terminate. Please check to see that your");
 puts("QSO.DAT file is not corrupted, or that it contains less than 1150");
 puts("QSO's. If this problem persists, or you need more than 1150 entries");
 puts("please email Tim Stoffel at lionlamb@powernet.net. Attach a copy of");
 puts("QSO.DAT file that is giving you problems.\n");
 }

if(errortype == 0 || errortype == 3 || errortype == 16 || errortype == 21) {
 puts("This may be a major program error. Please report this error to Tim");
 puts("Stoffel at lionlamb@powernet.net  .Please attach a copy of the");
 puts("QSO.DAT file that caused this error.\n");
 }
if(errortype == 1 || errortype == 11 || errortype == 12)
 return(0);
else
 return(1);

}
/* RECORD writes one character to the specified file, and does the error   */
/* trapping. */

int record(char ch, FILE *cabptr, struct qsorec *qrecptr)
{
fputc(ch, cabptr);
if(ferror(cabptr)) {
 warnmsg(24, 0);
 fclose(cabptr);
 free(qrecptr);
 exit(0);
 }
return(0);
}

/* RECSTR writes c characters of string 'works' to the specified file.    */
/* All necessary error trapping is taken care of. */

int recstr(char *workptr, int c, FILE *cabptr, struct qsorec *qrecptr)
{
int x;
for(x = 0; x <=(c-1); x++) {
 fputc(workptr[x], cabptr);
 if(ferror(cabptr)) {
  warnmsg(24, 0);
  fclose(cabptr);
  free(qrecptr);
  exit(0);
  }
 }
return(0);
}


/* CRLF writes a carriage return and a line feed to the specified file, */
/* and does the necessary error trapping. */

int crlf(FILE *cabptr, struct qsorec *qrecptr)
{
fputc(0x0d, cabptr);
if(ferror(cabptr)) {
 warnmsg(24, 0);
 fclose(cabptr);
 free(qrecptr);
 exit(0);
 }
fputc(0x0a, cabptr);
if(ferror(cabptr)) {
 warnmsg(24, 0);
 fclose(cabptr);
 free(qrecptr);
 exit(0);
 }
return(0);
}

/* Simple routine for clearing the screen. */

void clrscrn(void)
{
int x;
short row = 1, col = 1;
for(x = 0; x <= 25; x++)
 printf("\n");
 _settextposition(row, col);
 }


